/**
 * @function doCreateVerify
 * @returns {function}
 */
'use strict'

const co = require('co')
const uuid = require('uuid')
const successPayload = require('apeman-app-rest/lib/payloads/success_payload')
const failurePayload = require('apeman-app-rest/lib/payloads/failure_payload')
const apemanmodel = require('apemanmodel')
const validateDataType = require('apeman-app-rest/lib/validating/validate_data_type')
const errors = require('../errors')

/** @lends doCreateVerify */
function doCreateVerify (config) {
  const ACTION_NAME = 'create'
  let success = successPayload(ACTION_NAME)
  let failure = failurePayload(ACTION_NAME)

  let { expireDuration } = config

  return function createVerify (UserVerify, context) {
    const User = UserVerify.getRef('user').model

    let oneUser = apemanmodel.defineOne(User, context.transaction)
    let update = apemanmodel.defineUpdate(UserVerify, context.transaction)

    let { data, transaction, req } = context

    return co(function * () {
      let signed = req.signed()
      let hasSign = signed && signed.sign
      if (!hasSign) {
        return failure(errors.signinRequiredError())
      }

      let { userId } = signed.sign
      let user = yield oneUser(userId)
      if (!user) {
        return failure(
          errors.notFoundError(apemanmodel.toType(User), { id: userId })
        )
      }

      let typeError = validateDataType(data, apemanmodel.toType(UserVerify))
      if (typeError) {
        return failure(typeError)
      }

      let kind = data.attributes.kind
      let target = user[ kind ]
      if (!target) {
        // TODO Give more detailed error
        return failure(errors.somethingWrongError())
      }
      let token = uuid.v4().replace(/\-/g, '')
      let verifies = yield UserVerify.findOrCreate({
        transaction,
        where: { userId },
        defaults: { userId, kind, token, target }
      })
      let verify = [].concat(verifies || [])[ 0 ]
      let updated = yield update(verify, {
        token,
        kind,
        target,
        expireAt: new Date(new Date().getTime() + expireDuration),
        done: false
      })
      return success(
        {},
        apemanmodel.toData(UserVerify, updated)
      )
    })
  }
}

module.exports = doCreateVerify
