/**
 * Test case for create.
 * Runs with mocha.
 */
'use strict'

const create = require('../lib/create.js')
const apemanport = require('apemanport')
const co = require('co')
const apemanApp = require('apeman-app')
const fs = require('fs')
const apemandb = require('apemandb')
const apemanmodel = require('apemanmodel')
const apemanrequest = require('apemanrequest')
const assert = require('assert')

let tmpDir = `${__dirname}/../tmp`
let dbFile = tmpDir + '/testing-db01.db'

const loc = require('../doc/mocks/locales')

describe('create', () => {
  let user01, sign01, pass01
  let db, baseUrl, server, verify01
  let request = apemanrequest.create({ jar: true })

  before(() => co(function * () {
    db = apemandb({
      env: {
        DATABASE: 'testing-database02',
        USERNAME: 'FOO',
        PASSWORD: 'BAR',
        DIALECT: 'sqlite',
        STORAGE: dbFile,
        LOGGING: false
      },
      models: `${apemanmodel.paths.SAMPLE_DEFS_DIR}/*.json`
    })

    yield db.sync({ force: true })

    let port = yield apemanport.find()
    server = yield apemanApp('testing-app', {
      port,
      configuration: {
        $apps: {
          'testing-app': {
            '/': [
              require('apeman-app-json')(),
              require('apeman-app-format')(),
              require('apeman-app-query')(),
              require('apeman-app-send')(),
              require('apeman-app-header')(),
              require('apeman-app-form')(),
              require('apeman-app-link')(),
              require('apeman-app-lang')(),
              require('apeman-app-locale')(loc),
              require('apeman-app-jsonapi')(),
              require('apeman-app-session')(
                db.model('Session'), '1234qwer'
              ),
              require('apeman-app-signed')(),
              require('apeman-app-rest')(
                db.model('User'),
                {
                  pathname: '/api/users'
                }
              ),
              require('apeman-app-sign')(
                db.model('UserSign'),
                {
                  pathname: '/api/sign'
                }
              ),
              create(
                db.model('UserVerify'),
                {
                  pathname: '/api/verify'
                })
            ]
          }
        }
      }
    })
    baseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
    yield server.close()
    yield new Promise((resolve) =>
      setTimeout(() => {
        fs.unlink(dbFile, () => resolve())
      }, 100)
    ) // Wait for flush.
  }))

  it('Send fail.', () => co(function * () {
    let res = yield request({
      url: `${baseUrl}/api/verify`,
      method: 'POST',
      json: true,
      form: {
        data: {
          attributes: {
            kind: '__invalid_kind___'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 422)
    let errors = body.errors
    assert.equal(errors[ 0 ].source.pointer, '/data/attributes/kind')
  }))

  it('Create user', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/users`,
      json: true,
      form: {
        data: {
          type: 'users',
          attributes: {
            key: 'foo_bar',
            email: 'user01@exampole.com'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 201)
    let { data } = body
    user01 = Object.assign(
      {},
      data.attributes,
      { id: data.id }
    )
    assert.equal(user01.key, 'foo_bar')
    assert.equal(user01.email, 'user01@exampole.com')
  }))

  it('Create sign', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/sign`,
      json: true,
      form: {
        data: {
          type: 'user_signs',
          attributes: {
            key: user01.key,
            password: 'the-password-of-user01',
            kind: 'user.key'
          },
          relationships: {
            user: {
              data: {
                type: 'users',
                id: user01.id
              }
            }
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 201)
    let data = body.data
    assert.ok(data.attributes.signupAt)
    assert.ok(!data.attributes.hash)
    assert.ok(!data.attributes.hashSalt)
    sign01 = Object.assign({ id: data.id }, data.attributes)
    assert.ok(sign01)
  }))

  it('Create sign pass', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/sign/pass`,
      json: true,
      form: {
        data: {
          type: 'user_sign_passes',
          attributes: {
            key: sign01.key,
            password: 'the-password-of-user01-updated-bulk'
          }
        }
      }
    })
    let { body } = res
    let data = body.data
    assert.equal(data.type, 'user_sign_passes')
    assert.ok(data.relationships)
    assert.ok(data.relationships.sign)
    assert.ok(data.relationships.sign.data)
    pass01 = Object.assign({ id: data.id }, data.attributes)
  }))

  it('Create success.', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/verify`,
      json: true,
      form: {
        data: {
          type: 'user_verifies',
          attributes: {
            kind: 'email'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 200)
    assert.ok(body.meta.action, 'create')
    let data = body.data
    verify01 = Object.assign({}, data.attributes, { id: data.id })
  }))

  it('List verified to false', () => co(function * () {
    let res = yield request({
      method: 'GET',
      url: `${baseUrl}/api/verify?kind=email`,
      json: true
    })
    let { body } = res
    assert.equal(res.statusCode, 200)
    let verify = body.data[ 0 ]
    assert.equal(verify.attributes.done, false)
  }))

  it('Update Verify fail.', () => co(function * () {
    let res = yield request({
      method: 'PATCH',
      url: `${baseUrl}/api/verify/${verify01.id}`,
      json: true,
      form: {
        data: {
          type: 'user_verifies',
          attributes: {
            token: '__invalid_token__'
          }
        }
      }
    })
    assert.equal(res.statusCode, 401)
  }))

  it('Update Verify fail.', () => co(function * () {
    let res = yield request({
      method: 'PATCH',
      url: `${baseUrl}/api/verify/${verify01.id}`,
      json: true,
      form: {
        data: {
          type: 'user_verifies',
          attributes: {
            token: verify01.token
          }
        }
      }
    })
    assert.equal(res.statusCode, 200)
    let { body } = res
    assert.ok(body)
    assert.equal(body.meta.action, 'update')
    let data = body.data
    assert.equal(data.type, 'user_verifies')
  }))

  it('List verified to false', () => co(function * () {
    let res = yield request({
      method: 'GET',
      url: `${baseUrl}/api/verify?kind=email`,
      json: true
    })
    let { body } = res
    assert.equal(res.statusCode, 200)
    let verify = body.data[ 0 ]
    assert.equal(verify.attributes.done, true)
  }))
})

/* global describe, before, after, it */
